@@ -1,6 +1,7 @@
 import functools
 import json
 import re
+from types import TracebackType
 from typing import Any
 from typing import Callable
 from typing import Dict
@@ -8,6 +9,7 @@ from typing import List
 from typing import Literal
 from typing import Optional
 from typing import Set
+from typing import Tuple
 from typing import Union
 from urllib import parse
 
@@ -534,7 +536,7 @@ def end_context(span: Span):
         finalize_asm_env(env)
 
 
-def _on_context_ended(ctx):
+def _on_context_ended(ctx, _exc_info: Tuple[Optional[type], Optional[BaseException], Optional[TracebackType]]):
     env = ctx.get_local_item(_ASM_CONTEXT)
     if env is not None:
         finalize_asm_env(env)
