@@ -6,7 +6,6 @@ Django internals are instrumented via normal `patch()`.
 `django.apps.registry.Apps.populate` is patched to add instrumentation for any
 specific Django apps like Django Rest Framework (DRF).
 """
-
 from collections.abc import Iterable
 import functools
 from inspect import getmro
@@ -15,6 +14,7 @@ from inspect import isfunction
 from inspect import unwrap
 import os
 from typing import Dict
+from typing import cast
 
 import wrapt
 from wrapt.importer import when_imported
@@ -34,13 +34,13 @@ from ddtrace.ext import net
 from ddtrace.ext import sql as sqlx
 from ddtrace.internal import core
 from ddtrace.internal._exceptions import BlockingException
-from ddtrace.internal.compat import maybe_stringify
 from ddtrace.internal.constants import COMPONENT
 from ddtrace.internal.core.event_hub import ResultType
 from ddtrace.internal.logger import get_logger
 from ddtrace.internal.schema import schematize_service_name
 from ddtrace.internal.schema import schematize_url_operation
 from ddtrace.internal.schema.span_attribute_schema import SpanDirection
+from ddtrace.internal.telemetry import get_config as _get_config
 from ddtrace.internal.utils import get_argument_value
 from ddtrace.internal.utils import get_blocked
 from ddtrace.internal.utils import http as http_utils
@@ -49,6 +49,7 @@ from ddtrace.internal.utils.formats import asbool
 from ddtrace.internal.utils.importlib import func_name
 from ddtrace.propagation._database_monitoring import _DBM_Propagator
 from ddtrace.settings.asm import config as asm_config
+from ddtrace.settings.asm import endpoint_collection
 from ddtrace.settings.integration import IntegrationConfig
 from ddtrace.trace import Pin
 from ddtrace.vendor.packaging.version import parse as parse_version
@@ -79,10 +80,26 @@ config._add(
         ),
         use_handler_resource_format=asbool(os.getenv("DD_DJANGO_USE_HANDLER_RESOURCE_FORMAT", default=False)),
         use_legacy_resource_format=asbool(os.getenv("DD_DJANGO_USE_LEGACY_RESOURCE_FORMAT", default=False)),
-        _trace_asgi_websocket=os.getenv("DD_ASGI_TRACE_WEBSOCKET", default=False),
+        trace_asgi_websocket_messages=_get_config(
+            "DD_TRACE_WEBSOCKET_MESSAGES_ENABLED",
+            default=_get_config("DD_ASGI_TRACE_WEBSOCKET", default=False, modifier=asbool),
+            modifier=asbool,
+        ),
+        asgi_websocket_messages_inherit_sampling=asbool(
+            _get_config("DD_TRACE_WEBSOCKET_MESSAGES_INHERIT_SAMPLING", default=True)
+        )
+        and asbool(_get_config("DD_TRACE_WEBSOCKET_MESSAGES_SEPARATE_TRACES", default=True)),
+        websocket_messages_separate_traces=asbool(
+            _get_config("DD_TRACE_WEBSOCKET_MESSAGES_SEPARATE_TRACES", default=True)
+        ),
+        obfuscate_404_resource=os.getenv("DD_ASGI_OBFUSCATE_404_RESOURCE", default=False),
+        views={},
     ),
 )
 
+# PERF: cache the getattr lookup for the Django config
+config_django: IntegrationConfig = cast(IntegrationConfig, config.django)
+
 _NotSet = object()
 psycopg_cursor_cls = Psycopg2TracedCursor = Psycopg3TracedCursor = _NotSet
 
@@ -138,10 +155,9 @@ def patch_conn(django, conn):
     def cursor(django, pin, func, instance, args, kwargs):
         alias = getattr(conn, "alias", "default")
 
-        if config.django.database_service_name:
-            service = config.django.database_service_name
-        else:
-            database_prefix = config.django.database_service_name_prefix
+        service = config_django.database_service_name
+        if not service:
+            database_prefix = config_django.database_service_name_prefix
             service = "{}{}{}".format(database_prefix, alias, "db")
             service = schematize_service_name(service)
 
@@ -174,11 +190,11 @@ def patch_conn(django, conn):
 
         # Each db alias will need its own config for dbapi
         cfg = IntegrationConfig(
-            config.django.global_config,
-            "{}-{}".format("django", alias),  # name not used but set anyway
+            config_django.global_config,
+            "django-database",
             _default_service=config.django._default_service,
             _dbapi_span_name_prefix=prefix,
-            trace_fetch_methods=config.django.trace_fetch_methods,
+            trace_fetch_methods=config_django.trace_fetch_methods,
             _dbm_propagator=_DBM_Propagator(0, "query"),
         )
         return traced_cursor_cls(cursor, pin, cfg)
@@ -206,11 +222,11 @@ def instrument_dbs(django):
 def traced_cache(django, pin, func, instance, args, kwargs):
     from . import utils
 
-    if not config.django.instrument_caches:
+    if not config_django.instrument_caches:
         return func(*args, **kwargs)
 
     cache_backend = "{}.{}".format(instance.__module__, instance.__class__.__name__)
-    tags = {COMPONENT: config.django.integration_name, "django.cache.backend": cache_backend}
+    tags = {COMPONENT: config_django.integration_name, "django.cache.backend": cache_backend}
     if args:
         keys = utils.quantize_key_values(args[0])
         tags["django.cache.key"] = keys
@@ -219,7 +235,7 @@ def traced_cache(django, pin, func, instance, args, kwargs):
         "django.cache",
         span_name="django.cache",
         span_type=SpanTypes.CACHE,
-        service=schematize_service_name(config.django.cache_service_name),
+        service=schematize_service_name(config_django.cache_service_name),
         resource=utils.resource_from_cache_prefix(func_name(func), instance),
         tags=tags,
         pin=pin,
@@ -291,14 +307,14 @@ def traced_populate(django, pin, func, instance, args, kwargs):
     settings = django.conf.settings
 
     # Instrument databases
-    if config.django.instrument_databases:
+    if config_django.instrument_databases:
         try:
             instrument_dbs(django)
         except Exception:
             log.debug("Error instrumenting Django database connections", exc_info=True)
 
     # Instrument caches
-    if config.django.instrument_caches:
+    if config_django.instrument_caches:
         try:
             instrument_caches(django)
         except Exception:
@@ -320,7 +336,7 @@ def traced_populate(django, pin, func, instance, args, kwargs):
 
 def traced_func(django, name, resource=None, ignored_excs=None):
     def wrapped(django, pin, func, instance, args, kwargs):
-        tags = {COMPONENT: config.django.integration_name}
+        tags = {COMPONENT: config_django.integration_name}
         with core.context_with_data(
             "django.func.wrapped", span_name=name, resource=resource, tags=tags, pin=pin
         ) as ctx, ctx.span:
@@ -341,7 +357,7 @@ def traced_func(django, name, resource=None, ignored_excs=None):
 
 def traced_process_exception(django, name, resource=None):
     def wrapped(django, pin, func, instance, args, kwargs):
-        tags = {COMPONENT: config.django.integration_name}
+        tags = {COMPONENT: config_django.integration_name}
         with core.context_with_data(
             "django.process_exception", span_name=name, resource=resource, tags=tags, pin=pin
         ) as ctx, ctx.span:
@@ -434,14 +450,14 @@ def _gather_block_metadata(request, request_headers, ctx: core.ExecutionContext)
         metadata = {http.STATUS_CODE: "403", http.METHOD: request.method}
         url = utils.get_request_uri(request)
         query = request.META.get("QUERY_STRING", "")
-        if query and config.django.trace_query_string:
+        if query and config_django.trace_query_string:
             metadata[http.QUERY_STRING] = query
         user_agent = _get_request_header_user_agent(request_headers)
         if user_agent:
             metadata[http.USER_AGENT] = user_agent
     except Exception as e:
         log.warning("Could not gather some metadata on blocked request: %s", str(e))  # noqa: G200
-    core.dispatch("django.block_request_callback", (ctx, metadata, config.django, url, query))
+    core.dispatch("django.block_request_callback", (ctx, metadata, config_django, url, query))
 
 
 def _block_request_callable(request, request_headers, ctx: core.ExecutionContext):
@@ -480,10 +496,10 @@ def traced_get_response(django, pin, func, instance, args, kwargs):
         headers_case_sensitive=True,
         span_name=schematize_url_operation("django.request", protocol="http", direction=SpanDirection.INBOUND),
         resource=utils.REQUEST_DEFAULT_RESOURCE,
-        service=trace_utils.int_service(pin, config.django),
+        service=trace_utils.int_service(pin, config_django),
         span_type=SpanTypes.WEB,
-        tags={COMPONENT: config.django.integration_name, SPAN_KIND: SpanKind.SERVER},
-        integration_config=config.django,
+        tags={COMPONENT: config_django.integration_name, SPAN_KIND: SpanKind.SERVER},
+        integration_config=config_django,
         distributed_headers=request_headers,
         activate_distributed_headers=True,
         pin=pin,
@@ -567,37 +583,7 @@ def traced_get_response(django, pin, func, instance, args, kwargs):
                     return response  # noqa: B012
 
 
-@trace_utils.with_traced_module
-def traced_template_render(django, pin, wrapped, instance, args, kwargs):
-    # DEV: Check here in case this setting is configured after a template has been instrumented
-    if not config.django.instrument_templates:
-        return wrapped(*args, **kwargs)
-
-    template_name = maybe_stringify(getattr(instance, "name", None))
-    if template_name:
-        resource = template_name
-    else:
-        resource = "{0}.{1}".format(func_name(instance), wrapped.__name__)
-
-    tags = {COMPONENT: config.django.integration_name}
-    if template_name:
-        tags["django.template.name"] = template_name
-    engine = getattr(instance, "engine", None)
-    if engine:
-        tags["django.template.engine.class"] = func_name(engine)
-
-    with core.context_with_data(
-        "django.template.render",
-        span_name="django.template.render",
-        resource=resource,
-        span_type=http.TEMPLATE,
-        tags=tags,
-        pin=pin,
-    ) as ctx, ctx.span:
-        return wrapped(*args, **kwargs)
-
-
-def instrument_view(django, view):
+def instrument_view(django, view, path=None):
     """
     Helper to wrap Django views.
 
@@ -607,10 +593,24 @@ def instrument_view(django, view):
         for cls in reversed(getmro(view)):
             _instrument_view(django, cls)
 
-    return _instrument_view(django, view)
+    return _instrument_view(django, view, path=path)
 
 
-def _instrument_view(django, view):
+def extract_request_method_list(view):
+    try:
+        while "view_func" in view.__code__.co_freevars:
+            view = view.__closure__[view.__code__.co_freevars.index("view_func")].cell_contents
+        if "request_method_list" in view.__code__.co_freevars:
+            return view.__closure__[view.__code__.co_freevars.index("request_method_list")].cell_contents
+        return []
+    except Exception:
+        return []
+
+
+_DEFAULT_METHODS = ("get", "delete", "post", "options", "head")
+
+
+def _instrument_view(django, view, path=None):
     """Helper to wrap Django views."""
     from . import utils
 
@@ -619,9 +619,14 @@ def _instrument_view(django, view):
         return view
 
     # Patch view HTTP methods and lifecycle methods
-    http_method_names = getattr(view, "http_method_names", ("get", "delete", "post", "options", "head"))
+
+    http_method_names = getattr(view, "http_method_names", ())
+    request_method_list = extract_request_method_list(view) or http_method_names
+    if path is not None:
+        for method in request_method_list or ["*"]:
+            endpoint_collection.add_endpoint(method, path, operation_name="django.request")
     lifecycle_methods = ("setup", "dispatch", "http_method_not_allowed")
-    for name in list(http_method_names) + list(lifecycle_methods):
+    for name in list(request_method_list or _DEFAULT_METHODS) + list(lifecycle_methods):
         try:
             func = getattr(view, name, None)
             if not func or isinstance(func, wrapt.ObjectProxy):
@@ -662,20 +667,22 @@ def _instrument_view(django, view):
 def traced_urls_path(django, pin, wrapped, instance, args, kwargs):
     """Wrapper for url path helpers to ensure all views registered as urls are traced."""
     try:
-        from_args = False
-        view = kwargs.pop("view", None)
-        if view is None:
+        view_from_args = False
+        view = kwargs.get("view", None)
+        path = kwargs.get("route", None)
+        if view is None and len(args) > 1:
             view = args[1]
-            from_args = True
+            view_from_args = True
+        if path is None and args:
+            path = args[0]
 
         core.dispatch("service_entrypoint.patch", (unwrap(view),))
-
-        if from_args:
+        if view_from_args:
             args = list(args)
-            args[1] = instrument_view(django, view)
+            args[1] = instrument_view(django, view, path=path)
             args = tuple(args)
         else:
-            kwargs["view"] = instrument_view(django, view)
+            kwargs["view"] = instrument_view(django, view, path=path)
     except Exception:
         log.debug("Failed to instrument Django url path %r %r", args, kwargs, exc_info=True)
     return wrapped(*args, **kwargs)
@@ -714,11 +721,13 @@ def traced_technical_500_response(django, pin, func, instance, args, kwargs):
 @trace_utils.with_traced_module
 def traced_get_asgi_application(django, pin, func, instance, args, kwargs):
     from ddtrace.contrib.asgi import TraceMiddleware
+    from ddtrace.internal.constants import COMPONENT
 
     def django_asgi_modifier(span, scope):
         span.name = schematize_url_operation("django.request", protocol="http", direction=SpanDirection.INBOUND)
+        span.set_tag_str(COMPONENT, config_django.integration_name)
 
-    return TraceMiddleware(func(*args, **kwargs), integration_config=config.django, span_modifier=django_asgi_modifier)
+    return TraceMiddleware(func(*args, **kwargs), integration_config=config_django, span_modifier=django_asgi_modifier)
 
 
 class _DjangoUserInfoRetriever(_UserInfoRetriever):
@@ -806,7 +815,7 @@ def traced_login(django, pin, func, instance, args, kwargs):
     try:
         request = get_argument_value(args, kwargs, 0, "request")
         user = get_argument_value(args, kwargs, 1, "user")
-        core.dispatch("django.login", (pin, request, user, mode, _DjangoUserInfoRetriever(user), config.django))
+        core.dispatch("django.login", (pin, request, user, mode, _DjangoUserInfoRetriever(user), config_django))
     except Exception:
         log.debug("Error while trying to trace Django login", exc_info=True)
 
@@ -820,7 +829,7 @@ def traced_authenticate(django, pin, func, instance, args, kwargs):
     try:
         result = core.dispatch_with_results(
             "django.auth",
-            (result_user, mode, kwargs, pin, _DjangoUserInfoRetriever(result_user, credentials=kwargs), config.django),
+            (result_user, mode, kwargs, pin, _DjangoUserInfoRetriever(result_user, credentials=kwargs), config_django),
         ).user
         if result and result.value[0]:
             return result.value[1]
@@ -832,7 +841,7 @@ def traced_authenticate(django, pin, func, instance, args, kwargs):
 
 @trace_utils.with_traced_module
 def traced_process_request(django, pin, func, instance, args, kwargs):
-    tags = {COMPONENT: config.django.integration_name}
+    tags = {COMPONENT: config_django.integration_name}
     with core.context_with_data(
         "django.func.wrapped",
         span_name="django.middleware",
@@ -875,7 +884,7 @@ def traced_process_request(django, pin, func, instance, args, kwargs):
                         kwargs,
                         pin,
                         _DjangoUserInfoRetriever(request_user, credentials=kwargs),
-                        config.django,
+                        config_django,
                     ),
                 )
         except Exception:
@@ -886,7 +895,7 @@ def traced_process_request(django, pin, func, instance, args, kwargs):
 def patch_create_user(django, pin, func, instance, args, kwargs):
     user = func(*args, **kwargs)
     core.dispatch(
-        "django.create_user", (config.django, pin, func, instance, args, kwargs, user, _DjangoUserInfoRetriever(user))
+        "django.create_user", (config_django, pin, func, instance, args, kwargs, user, _DjangoUserInfoRetriever(user))
     )
     return user
 
@@ -919,7 +928,7 @@ def _patch(django):
 
     when_imported("django.apps.registry")(lambda m: trace_utils.wrap(m, "Apps.populate", traced_populate(django)))
 
-    if config.django.instrument_middleware:
+    if config_django.instrument_middleware:
         when_imported("django.core.handlers.base")(
             lambda m: trace_utils.wrap(m, "BaseHandler.load_middleware", traced_load_middleware(django))
         )
@@ -955,10 +964,10 @@ def _patch(django):
             lambda m: trace_utils.wrap(m, "get_asgi_application", traced_get_asgi_application(django))
         )
 
-    if config.django.instrument_templates:
-        when_imported("django.template.base")(
-            lambda m: trace_utils.wrap(m, "Template.render", traced_template_render(django))
-        )
+    if config_django.instrument_templates:
+        from .templates import DjangoTemplateWrappingContext
+
+        when_imported("django.template.base")(DjangoTemplateWrappingContext.instrument_module)
 
     if django.VERSION < (4, 0, 0):
         when_imported("django.conf.urls")(lambda m: trace_utils.wrap(m, "url", traced_urls_path(django)))
@@ -1031,6 +1040,11 @@ def _unpatch(django):
         trace_utils.unwrap(conn, "cursor")
     trace_utils.unwrap(django.db.utils.ConnectionHandler, "__getitem__")
 
+    if config.django.instrument_templates:
+        from .templates import DjangoTemplateWrappingContext
+
+        DjangoTemplateWrappingContext.uninstrument_module(django.template.base)
+
 
 def unpatch():
     import django
