@@ -1,5 +1,6 @@
 from ddtrace._trace.processor import TraceProcessor
 from ddtrace.constants import _BASE_SERVICE_KEY
+from ddtrace.internal.serverless import in_aws_lambda
 from ddtrace.settings._config import config
 
 from . import schematize_service_name
@@ -8,10 +9,13 @@ from . import schematize_service_name
 class BaseServiceProcessor(TraceProcessor):
     def __init__(self):
         self._global_service = schematize_service_name((config.service or "").lower())
+        self._in_aws_lambda = in_aws_lambda()
 
     def process_trace(self, trace):
-        if not trace:
-            return
+        # AWS Lambda spans receive unhelpful base_service value of runtime
+        # Remove base_service to prevent service overrides in Lambda spans
+        if not trace or self._in_aws_lambda:
+            return trace
 
         traces_to_process = filter(
             lambda x: x.service and x.service.lower() != self._global_service,
