@@ -7,6 +7,7 @@ from .google_genai import GoogleGenAIIntegration
 from .langchain import LangChainIntegration
 from .litellm import LiteLLMIntegration
 from .openai import OpenAIIntegration
+from .pydantic_ai import PydanticAIIntegration
 from .vertexai import VertexAIIntegration
 
 
@@ -20,5 +21,6 @@ __all__ = [
     "LangChainIntegration",
     "LiteLLMIntegration",
     "OpenAIIntegration",
+    "PydanticAIIntegration",
     "VertexAIIntegration",
 ]
