@@ -1,7 +1,6 @@
-from dataclasses import asdict
 from dataclasses import dataclass
-from dataclasses import is_dataclass
 import json
+import re
 from typing import Any
 from typing import Dict
 from typing import List
@@ -18,7 +17,6 @@ from ddtrace.llmobs._constants import DISPATCH_ON_TOOL_CALL_OUTPUT_USED
 from ddtrace.llmobs._constants import INPUT_MESSAGES
 from ddtrace.llmobs._constants import INPUT_TOKENS_METRIC_KEY
 from ddtrace.llmobs._constants import INPUT_VALUE
-from ddtrace.llmobs._constants import LITELLM_ROUTER_INSTANCE_KEY
 from ddtrace.llmobs._constants import METADATA
 from ddtrace.llmobs._constants import OAI_HANDOFF_TOOL_ARG
 from ddtrace.llmobs._constants import OUTPUT_MESSAGES
@@ -26,140 +24,123 @@ from ddtrace.llmobs._constants import OUTPUT_TOKENS_METRIC_KEY
 from ddtrace.llmobs._constants import OUTPUT_VALUE
 from ddtrace.llmobs._constants import TOTAL_TOKENS_METRIC_KEY
 from ddtrace.llmobs._utils import _get_attr
+from ddtrace.llmobs._utils import load_data_value
 from ddtrace.llmobs._utils import safe_json
 
 
+try:
+    from tiktoken import encoding_for_model
+
+    tiktoken_available = True
+except ModuleNotFoundError:
+    tiktoken_available = False
+
 logger = get_logger(__name__)
 
-OPENAI_SKIPPED_COMPLETION_TAGS = (
-    "model",
+COMMON_METADATA_KEYS = (
+    "stream",
+    "temperature",
+    "top_p",
+    "user",
+)
+OPENAI_METADATA_RESPONSE_KEYS = (
+    "background",
+    "include",
+    "max_output_tokens",
+    "max_tool_calls",
+    "parallel_tool_calls",
+    "previous_response_id",
     "prompt",
-    "api_key",
-    "user_api_key",
-    "user_api_key_hash",
-    LITELLM_ROUTER_INSTANCE_KEY,
-)
-OPENAI_SKIPPED_CHAT_TAGS = (
-    "model",
-    "messages",
+    "reasoning",
+    "service_tier",
+    "store",
+    "text",
+    "tool_choice",
     "tools",
-    "functions",
-    "api_key",
-    "user_api_key",
-    "user_api_key_hash",
-    LITELLM_ROUTER_INSTANCE_KEY,
+    "top_logprobs",
+    "truncation",
+)
+OPENAI_METADATA_CHAT_KEYS = (
+    "audio",
+    "frequency_penalty",
+    "function_call",
+    "logit_bias",
+    "logprobs",
+    "max_completion_tokens",
+    "max_tokens",
+    "modalities",
+    "n",
+    "parallel_tool_calls",
+    "prediction",
+    "presence_penalty",
+    "reasoning_effort",
+    "response_format",
+    "seed",
+    "service_tier",
+    "stop",
+    "store",
+    "stream_options",
+    "tool_choice",
+    "top_logprobs",
+    "web_search_options",
+)
+OPENAI_METADATA_COMPLETION_KEYS = (
+    "best_of",
+    "echo",
+    "frequency_penalty",
+    "logit_bias",
+    "logprobs",
+    "max_tokens",
+    "n",
+    "presence_penalty",
+    "seed",
+    "stop",
+    "stream_options",
+    "suffix",
 )
 
-
-def extract_model_name_google(instance, model_name_attr):
-    """Extract the model name from the instance.
-    Model names are stored in the format `"models/{model_name}"`
-    so we do our best to return the model name instead of the full string.
-    """
-    model_name = _get_attr(instance, model_name_attr, "")
-    if not model_name or not isinstance(model_name, str):
-        return ""
-    if "/" in model_name:
-        return model_name.split("/")[-1]
-    return model_name
-
-
-def get_generation_config_google(instance, kwargs):
-    """
-    The generation config can be defined on the model instance or
-    as a kwarg of the request. Therefore, try to extract this information
-    from the kwargs and otherwise default to checking the model instance attribute.
-    """
-    generation_config = kwargs.get("generation_config", {})
-    return generation_config or _get_attr(instance, "_generation_config", {})
-
-
-def tag_request_content_part_google(tag_prefix, span, integration, part, part_idx, content_idx):
-    """Tag the generation span with request content parts."""
-    text = _get_attr(part, "text", "")
-    function_call = _get_attr(part, "function_call", None)
-    function_response = _get_attr(part, "function_response", None)
-    span.set_tag_str(
-        "%s.request.contents.%d.parts.%d.text" % (tag_prefix, content_idx, part_idx), integration.trunc(str(text))
-    )
-    if function_call:
-        function_call_dict = type(function_call).to_dict(function_call)
-        span.set_tag_str(
-            "%s.request.contents.%d.parts.%d.function_call.name" % (tag_prefix, content_idx, part_idx),
-            function_call_dict.get("name", ""),
-        )
-        span.set_tag_str(
-            "%s.request.contents.%d.parts.%d.function_call.args" % (tag_prefix, content_idx, part_idx),
-            integration.trunc(str(function_call_dict.get("args", {}))),
-        )
-    if function_response:
-        function_response_dict = type(function_response).to_dict(function_response)
-        span.set_tag_str(
-            "%s.request.contents.%d.parts.%d.function_response.name" % (tag_prefix, content_idx, part_idx),
-            function_response_dict.get("name", ""),
-        )
-        span.set_tag_str(
-            "%s.request.contents.%d.parts.%d.function_response.response" % (tag_prefix, content_idx, part_idx),
-            integration.trunc(str(function_response_dict.get("response", {}))),
-        )
-
-
-def tag_response_part_google(tag_prefix, span, integration, part, part_idx, candidate_idx):
-    """Tag the generation span with response part text and function calls."""
-    text = _get_attr(part, "text", "")
-    span.set_tag_str(
-        "%s.response.candidates.%d.content.parts.%d.text" % (tag_prefix, candidate_idx, part_idx),
-        integration.trunc(str(text)),
-    )
-    function_call = _get_attr(part, "function_call", None)
-    if not function_call:
-        return
-    span.set_tag_str(
-        "%s.response.candidates.%d.content.parts.%d.function_call.name" % (tag_prefix, candidate_idx, part_idx),
-        _get_attr(function_call, "name", ""),
-    )
-    span.set_tag_str(
-        "%s.response.candidates.%d.content.parts.%d.function_call.args" % (tag_prefix, candidate_idx, part_idx),
-        integration.trunc(str(_get_attr(function_call, "args", {}))),
-    )
-
-
-def llmobs_get_metadata_google(kwargs, instance):
-    metadata = {}
-    model_config = getattr(instance, "_generation_config", {}) or {}
-    model_config = model_config.to_dict() if hasattr(model_config, "to_dict") else model_config
-    request_config = kwargs.get("generation_config", {}) or {}
-    request_config = request_config.to_dict() if hasattr(request_config, "to_dict") else request_config
-
-    parameters = ("temperature", "max_output_tokens", "candidate_count", "top_p", "top_k")
-    for param in parameters:
-        model_config_value = _get_attr(model_config, param, None)
-        request_config_value = _get_attr(request_config, param, None)
-        if model_config_value or request_config_value:
-            metadata[param] = request_config_value or model_config_value
-    return metadata
-
-
-def extract_message_from_part_google(part, role=None):
-    text = _get_attr(part, "text", "")
-    function_call = _get_attr(part, "function_call", None)
-    function_response = _get_attr(part, "function_response", None)
-    message = {"content": text}
-    if role:
-        message["role"] = role
-    if function_call:
-        function_call_dict = function_call
-        if not isinstance(function_call, dict):
-            function_call_dict = type(function_call).to_dict(function_call)
-        message["tool_calls"] = [
-            {"name": function_call_dict.get("name", ""), "arguments": function_call_dict.get("args", {})}
-        ]
-    if function_response:
-        function_response_dict = function_response
-        if not isinstance(function_response, dict):
-            function_response_dict = type(function_response).to_dict(function_response)
-        message["content"] = "[tool result: {}]".format(function_response_dict.get("response", ""))
-    return message
+LITELLM_METADATA_CHAT_KEYS = (
+    "timeout",
+    "n",
+    "stream_options",
+    "stop",
+    "max_completion_tokens",
+    "max_tokens",
+    "modalities",
+    "prediction",
+    "presence_penalty",
+    "frequency_penalty",
+    "logit_bias",
+    "response_format",
+    "seed",
+    "tool_choice",
+    "parallel_tool_calls",
+    "logprobs",
+    "top_logprobs",
+    "deployment_id",
+    "reasoning_effort",
+    "base_url",
+    "api_base",
+    "api_version",
+    "model_list",
+)
+LITELLM_METADATA_COMPLETION_KEYS = (
+    "best_of",
+    "echo",
+    "frequency_penalty",
+    "logit_bias",
+    "logprobs",
+    "max_tokens",
+    "n",
+    "presence_penalty",
+    "stop",
+    "stream_options",
+    "suffix",
+    "api_base",
+    "api_version",
+    "model_list",
+    "custom_llm_provider",
+)
 
 
 def get_llmobs_metrics_tags(integration_name, span):
@@ -186,39 +167,18 @@ def get_llmobs_metrics_tags(integration_name, span):
     return usage
 
 
-def get_system_instructions_from_google_model(model_instance):
-    """
-    Extract system instructions from model and convert to []str for tagging.
-    """
-    try:
-        from google.ai.generativelanguage_v1beta.types.content import Content
-    except ImportError:
-        Content = None
-    try:
-        from vertexai.generative_models._generative_models import Part
-    except ImportError:
-        Part = None
-
-    raw_system_instructions = getattr(model_instance, "_system_instruction", [])
-    if Content is not None and isinstance(raw_system_instructions, Content):
-        system_instructions = []
-        for part in raw_system_instructions.parts:
-            system_instructions.append(_get_attr(part, "text", ""))
-        return system_instructions
-    elif isinstance(raw_system_instructions, str):
-        return [raw_system_instructions]
-    elif Part is not None and isinstance(raw_system_instructions, Part):
-        return [_get_attr(raw_system_instructions, "text", "")]
-    elif not isinstance(raw_system_instructions, list):
-        return []
-
-    system_instructions = []
-    for elem in raw_system_instructions:
-        if isinstance(elem, str):
-            system_instructions.append(elem)
-        elif Part is not None and isinstance(elem, Part):
-            system_instructions.append(_get_attr(elem, "text", ""))
-    return system_instructions
+def parse_llmobs_metric_args(metrics):
+    usage = {}
+    input_tokens = _get_attr(metrics, "prompt_tokens", None)
+    output_tokens = _get_attr(metrics, "completion_tokens", None)
+    total_tokens = _get_attr(metrics, "total_tokens", None)
+    if input_tokens is not None:
+        usage[INPUT_TOKENS_METRIC_KEY] = input_tokens
+    if output_tokens is not None:
+        usage[OUTPUT_TOKENS_METRIC_KEY] = output_tokens
+    if total_tokens is not None:
+        usage[TOTAL_TOKENS_METRIC_KEY] = total_tokens
+    return usage
 
 
 LANGCHAIN_ROLE_MAPPING = {
@@ -259,7 +219,7 @@ def get_content_from_langchain_message(message) -> Union[str, Tuple[str, str]]:
         return str(message)
 
 
-def get_messages_from_converse_content(role: str, content: list):
+def get_messages_from_converse_content(role: str, content: List[Dict[str, Any]]):
     """
     Extracts out a list of messages from a converse `content` field.
 
@@ -271,9 +231,11 @@ def get_messages_from_converse_content(role: str, content: list):
     """
     if not content or not isinstance(content, list) or not isinstance(content[0], dict):
         return []
-    messages = []  # type: list[dict[str, Union[str, list[dict[str, dict]]]]]
+    messages: List[Dict[str, Union[str, List[Dict[str, Any]]]]] = []
     content_blocks = []
     tool_calls_info = []
+    tool_messages: List[Dict[str, Any]] = []
+    unsupported_content_messages: List[Dict[str, Union[str, List[Dict[str, Any]]]]] = []
     for content_block in content:
         if content_block.get("text") and isinstance(content_block.get("text"), str):
             content_blocks.append(content_block.get("text", ""))
@@ -286,9 +248,29 @@ def get_messages_from_converse_content(role: str, content: list):
                     "tool_id": str(toolUse.get("toolUseId", "")),
                 }
             )
+        elif content_block.get("toolResult") and isinstance(content_block.get("toolResult"), dict):
+            tool_message: Dict[str, Any] = content_block.get("toolResult", {})
+            tool_message_contents: List[Dict[str, Any]] = tool_message.get("content", [])
+            tool_message_id: str = tool_message.get("toolUseId", "")
+
+            for tool_message_content in tool_message_contents:
+                tool_message_content_text: Optional[str] = tool_message_content.get("text")
+                tool_message_content_json: Optional[Dict[str, Any]] = tool_message_content.get("json")
+
+                tool_messages.append(
+                    {
+                        "content": tool_message_content_text
+                        or (tool_message_content_json and safe_json(tool_message_content_json))
+                        or f"[Unsupported content type(s): {','.join(tool_message_content.keys())}]",
+                        "role": "tool",
+                        "tool_id": tool_message_id,
+                    }
+                )
         else:
             content_type = ",".join(content_block.keys())
-            messages.append({"content": "[Unsupported content type: {}]".format(content_type), "role": role})
+            unsupported_content_messages.append(
+                {"content": "[Unsupported content type: {}]".format(content_type), "role": role}
+            )
     message = {}  # type: dict[str, Union[str, list[dict[str, dict]]]]
     if tool_calls_info:
         message.update({"tool_calls": tool_calls_info})
@@ -296,15 +278,21 @@ def get_messages_from_converse_content(role: str, content: list):
         message.update({"content": " ".join(content_blocks), "role": role})
     if message:
         messages.append(message)
+    if unsupported_content_messages:
+        messages.extend(unsupported_content_messages)
+    if tool_messages:
+        messages.extend(tool_messages)
     return messages
 
 
-def openai_set_meta_tags_from_completion(span: Span, kwargs: Dict[str, Any], completions: Any) -> None:
+def openai_set_meta_tags_from_completion(
+    span: Span, kwargs: Dict[str, Any], completions: Any, integration_name: str = "openai"
+) -> None:
     """Extract prompt/response tags from a completion and set them as temporary "_ml_obs.meta.*" tags."""
     prompt = kwargs.get("prompt", "")
     if isinstance(prompt, str):
         prompt = [prompt]
-    parameters = {k: v for k, v in kwargs.items() if k not in OPENAI_SKIPPED_COMPLETION_TAGS}
+    parameters = get_metadata_from_kwargs(kwargs, integration_name, "completion")
     output_messages = [{"content": ""}]
     if not span.error and completions:
         choices = getattr(completions, "choices", completions)
@@ -318,15 +306,33 @@ def openai_set_meta_tags_from_completion(span: Span, kwargs: Dict[str, Any], com
     )
 
 
-def openai_set_meta_tags_from_chat(span: Span, kwargs: Dict[str, Any], messages: Optional[Any]) -> None:
+def openai_set_meta_tags_from_chat(
+    span: Span, kwargs: Dict[str, Any], messages: Optional[Any], integration_name: str = "openai"
+) -> None:
     """Extract prompt/response tags from a chat completion and set them as temporary "_ml_obs.meta.*" tags."""
     input_messages = []
     for m in kwargs.get("messages", []):
-        tool_call_id = m.get("tool_call_id")
+        processed_message = {
+            "content": str(_get_attr(m, "content", "")),
+            "role": str(_get_attr(m, "role", "")),
+        }  # type: dict[str, Union[str, list[dict[str, dict]]]]
+        tool_call_id = _get_attr(m, "tool_call_id", None)
         if tool_call_id:
             core.dispatch(DISPATCH_ON_TOOL_CALL_OUTPUT_USED, (tool_call_id, span))
-        input_messages.append({"content": str(_get_attr(m, "content", "")), "role": str(_get_attr(m, "role", ""))})
-    parameters = {k: v for k, v in kwargs.items() if k not in OPENAI_SKIPPED_CHAT_TAGS}
+            processed_message["tool_id"] = tool_call_id
+        tool_calls = _get_attr(m, "tool_calls", [])
+        if tool_calls:
+            processed_message["tool_calls"] = [
+                {
+                    "name": _get_attr(_get_attr(tool_call, "function", {}), "name", ""),
+                    "arguments": _get_attr(_get_attr(tool_call, "function", {}), "arguments", {}),
+                    "tool_id": _get_attr(tool_call, "id", ""),
+                    "type": _get_attr(tool_call, "type", ""),
+                }
+                for tool_call in tool_calls
+            ]
+        input_messages.append(processed_message)
+    parameters = get_metadata_from_kwargs(kwargs, integration_name, "chat")
     span._set_ctx_items({INPUT_MESSAGES: input_messages, METADATA: parameters})
 
     if span.error or not messages:
@@ -398,6 +404,19 @@ def openai_set_meta_tags_from_chat(span: Span, kwargs: Dict[str, Any], messages:
     span._set_ctx_item(OUTPUT_MESSAGES, output_messages)
 
 
+def get_metadata_from_kwargs(
+    kwargs: Dict[str, Any], integration_name: str = "openai", operation: str = "chat"
+) -> Dict[str, Any]:
+    metadata = {}
+    keys_to_include: Tuple[str, ...] = COMMON_METADATA_KEYS
+    if integration_name == "openai":
+        keys_to_include += OPENAI_METADATA_CHAT_KEYS if operation == "chat" else OPENAI_METADATA_COMPLETION_KEYS
+    elif integration_name == "litellm":
+        keys_to_include += LITELLM_METADATA_CHAT_KEYS if operation == "chat" else LITELLM_METADATA_COMPLETION_KEYS
+    metadata = {k: v for k, v in kwargs.items() if k in keys_to_include}
+    return metadata
+
+
 def openai_get_input_messages_from_response_input(
     messages: Optional[Union[str, List[Dict[str, Any]]]]
 ) -> List[Dict[str, Any]]:
@@ -449,11 +468,9 @@ def openai_get_input_messages_from_response_input(
             # Process `FunctionCallOutput` type from input messages
             output = item["output"]
 
-            if isinstance(output, str):
-                try:
-                    output = json.loads(output)
-                except json.JSONDecodeError:
-                    output = {"output": str(output)}
+            if not isinstance(output, str):
+                output = safe_json(output)
+
             processed_item.update(
                 {
                     "role": "tool",
@@ -472,42 +489,45 @@ def openai_get_output_messages_from_response(response: Optional[Any]) -> List[Di
     Parses the output to openai responses api into a list of output messages
 
     Args:
-        response: An OpenAI response object containing output messages
+        response: An OpenAI response object or dictionary containing output messages
 
     Returns:
         - A list of processed messages
     """
-    if not response or not getattr(response, "output", None):
+    if not response:
+        return []
+
+    messages = _get_attr(response, "output", [])
+    if not messages:
         return []
 
-    messages: List[Any] = response.output
     processed: List[Dict[str, Any]] = []
 
     for item in messages:
         message = {}
-        message_type = getattr(item, "type", "")
+        message_type = _get_attr(item, "type", "")
 
         if message_type == "message":
             text = ""
-            for content in getattr(item, "content", []):
-                text += str(getattr(content, "text", "") or "")
-                text += str(getattr(content, "refusal", "") or "")
-            message.update({"role": getattr(item, "role", "assistant"), "content": text})
+            for content in _get_attr(item, "content", []):
+                text += str(_get_attr(content, "text", "") or "")
+                text += str(_get_attr(content, "refusal", "") or "")
+            message.update({"role": _get_attr(item, "role", "assistant"), "content": text})
         elif message_type == "reasoning":
             message.update(
                 {
                     "role": "reasoning",
                     "content": safe_json(
                         {
-                            "summary": getattr(item, "summary", ""),
-                            "encrypted_content": getattr(item, "encrypted_content", ""),
-                            "id": getattr(item, "id", ""),
+                            "summary": _get_attr(item, "summary", ""),
+                            "encrypted_content": _get_attr(item, "encrypted_content", ""),
+                            "id": _get_attr(item, "id", ""),
                         }
                     ),
                 }
             )
         elif message_type == "function_call":
-            arguments = getattr(item, "arguments", "{}")
+            arguments = _get_attr(item, "arguments", "{}")
             try:
                 arguments = json.loads(arguments)
             except json.JSONDecodeError:
@@ -516,10 +536,10 @@ def openai_get_output_messages_from_response(response: Optional[Any]) -> List[Di
                 {
                     "tool_calls": [
                         {
-                            "tool_id": getattr(item, "call_id", ""),
+                            "tool_id": _get_attr(item, "call_id", ""),
                             "arguments": arguments,
-                            "name": getattr(item, "name", ""),
-                            "type": getattr(item, "type", "function"),
+                            "name": _get_attr(item, "name", ""),
+                            "type": _get_attr(item, "type", "function"),
                         }
                     ]
                 }
@@ -538,7 +558,7 @@ def openai_get_metadata_from_response(
     metadata = {}
 
     if kwargs:
-        metadata.update({k: v for k, v in kwargs.items() if k not in ("model", "input", "instructions")})
+        metadata.update({k: v for k, v in kwargs.items() if k in OPENAI_METADATA_RESPONSE_KEYS + COMMON_METADATA_KEYS})
 
     if not response:
         return metadata
@@ -547,7 +567,7 @@ def openai_get_metadata_from_response(
     for field in ["temperature", "max_output_tokens", "top_p", "tools", "tool_choice", "truncation", "text", "user"]:
         value = getattr(response, field, None)
         if value is not None:
-            metadata[field] = load_oai_span_data_value(value)
+            metadata[field] = load_data_value(value)
 
     usage = getattr(response, "usage", None)
     output_tokens_details = getattr(usage, "output_tokens_details", None)
@@ -630,24 +650,24 @@ def openai_construct_message_from_streamed_chunks(streamed_chunks: List[Any]) ->
     """
     message: Dict[str, Any] = {"content": "", "tool_calls": []}
     for chunk in streamed_chunks:
-        if getattr(chunk, "usage", None):
+        if _get_attr(chunk, "usage", None):
             message["usage"] = chunk.usage
-        if not hasattr(chunk, "delta"):
+        if not _get_attr(chunk, "delta", None):
             continue
-        if getattr(chunk, "index", None) and not message.get("index"):
+        if _get_attr(chunk, "index", None) and not message.get("index"):
             message["index"] = chunk.index
-        if getattr(chunk.delta, "role") and not message.get("role"):
+        if _get_attr(chunk.delta, "role", None) and not message.get("role"):
             message["role"] = chunk.delta.role
-        if getattr(chunk, "finish_reason", None) and not message.get("finish_reason"):
+        if _get_attr(chunk, "finish_reason", None) and not message.get("finish_reason"):
             message["finish_reason"] = chunk.finish_reason
-        chunk_content = getattr(chunk.delta, "content", "")
+        chunk_content = _get_attr(chunk.delta, "content", "")
         if chunk_content:
             message["content"] += chunk_content
             continue
-        function_call = getattr(chunk.delta, "function_call", None)
+        function_call = _get_attr(chunk.delta, "function_call", None)
         if function_call:
             openai_construct_tool_call_from_streamed_chunk(message["tool_calls"], function_call_chunk=function_call)
-        tool_calls = getattr(chunk.delta, "tool_calls", None)
+        tool_calls = _get_attr(chunk.delta, "tool_calls", None)
         if not tool_calls:
             continue
         for tool_call in tool_calls:
@@ -780,7 +800,7 @@ class OaiSpanAdapter:
         data = self.data
         if not data:
             return {}
-        return load_oai_span_data_value(data)
+        return load_data_value(data)
 
     @property
     def response_output_text(self) -> str:
@@ -839,25 +859,14 @@ class OaiSpanAdapter:
                 if hasattr(self.response, field):
                     value = getattr(self.response, field)
                     if value is not None:
-                        metadata[field] = load_oai_span_data_value(value)
+                        metadata[field] = load_data_value(value)
 
             if hasattr(self.response, "text") and self.response.text:
-                metadata["text"] = load_oai_span_data_value(self.response.text)
+                metadata["text"] = load_data_value(self.response.text)
 
             if hasattr(self.response, "usage") and hasattr(self.response.usage, "output_tokens_details"):
                 metadata["reasoning_tokens"] = self.response.usage.output_tokens_details.reasoning_tokens
 
-        if self.span_type == "agent":
-            agent_metadata: Dict[str, List[str]] = {
-                "handoffs": [],
-                "tools": [],
-            }
-            if self.handoffs:
-                agent_metadata["handoffs"] = load_oai_span_data_value(self.handoffs)
-            if self.tools:
-                agent_metadata["tools"] = load_oai_span_data_value(self.tools)
-            metadata.update(agent_metadata)
-
         if self.span_type == "custom" and hasattr(self._raw_oai_span.span_data, "data"):
             custom_data = getattr(self._raw_oai_span.span_data, "data", None)
             if custom_data:
@@ -1070,22 +1079,6 @@ class OaiTraceAdapter:
         return self._trace
 
 
-def load_oai_span_data_value(value):
-    """Helper function to load values stored in openai span data in a consistent way"""
-    if isinstance(value, list):
-        return [load_oai_span_data_value(item) for item in value]
-    elif hasattr(value, "model_dump"):
-        return value.model_dump()
-    elif is_dataclass(value):
-        return asdict(value)
-    else:
-        value_str = safe_json(value)
-        try:
-            return json.loads(value_str)
-        except json.JSONDecodeError:
-            return value_str
-
-
 @dataclass
 class LLMObsTraceInfo:
     """Metadata for llmobs trace used for setting root span attributes and span links"""
@@ -1143,3 +1136,88 @@ def get_final_message_converse_stream_message(
         message_output["tool_calls"] = tool_calls
 
     return message_output
+
+
+_punc_regex = re.compile(r"[\w']+|[.,!?;~@#$%^&*()+/-]")
+
+
+def _compute_prompt_tokens(model_name, prompts=None, messages=None):
+    """Compute token span metrics on streamed chat/completion requests.
+    Only required if token usage is not provided in the streamed response.
+    """
+    num_prompt_tokens = 0
+    estimated = False
+    if messages:
+        for m in messages:
+            estimated, prompt_tokens = _compute_token_count(m.get("content", ""), model_name)
+            num_prompt_tokens += prompt_tokens
+    elif prompts:
+        if isinstance(prompts, str) or isinstance(prompts, list) and isinstance(prompts[0], int):
+            prompts = [prompts]
+        for prompt in prompts:
+            estimated, prompt_tokens = _compute_token_count(prompt, model_name)
+            num_prompt_tokens += prompt_tokens
+    return estimated, num_prompt_tokens
+
+
+def _compute_completion_tokens(completions_or_messages, model_name):
+    """Compute/Estimate the completion token count from the streamed response."""
+    if not completions_or_messages:
+        return False, 0
+    estimated = False
+    num_completion_tokens = 0
+    for choice in completions_or_messages:
+        content = choice.get("content", "") or choice.get("text", "")
+        estimated, completion_tokens = _compute_token_count(content, model_name)
+        num_completion_tokens += completion_tokens
+    return estimated, num_completion_tokens
+
+
+def _compute_token_count(content, model):
+    # type: (Union[str, List[int]], Optional[str]) -> Tuple[bool, int]
+    """
+    Takes in prompt/response(s) and model pair, and returns a tuple of whether or not the number of prompt
+    tokens was estimated, and the estimated/calculated prompt token count.
+    """
+    num_prompt_tokens = 0
+    estimated = False
+    if model is not None and tiktoken_available is True:
+        try:
+            enc = encoding_for_model(model)
+            if isinstance(content, str):
+                num_prompt_tokens += len(enc.encode(content))
+            elif content and isinstance(content, list) and isinstance(content[0], int):
+                num_prompt_tokens += len(content)
+            return estimated, num_prompt_tokens
+        except KeyError:
+            # tiktoken.encoding_for_model() will raise a KeyError if it doesn't have a tokenizer for the model
+            estimated = True
+    else:
+        estimated = True
+
+    # If model is unavailable or tiktoken is not imported, then provide a very rough estimate of the number of tokens
+    return estimated, _est_tokens(content)
+
+
+def _est_tokens(prompt):
+    # type: (Union[str, List[int]]) -> int
+    """
+    Provide a very rough estimate of the number of tokens in a string prompt.
+    Note that if the prompt is passed in as a token array (list of ints), the token count
+    is just the length of the token array.
+    """
+    # If model is unavailable or tiktoken is not imported, then provide a very rough estimate of the number of tokens
+    # Approximate using the following assumptions:
+    #    * English text
+    #    * 1 token ~= 4 chars
+    #    * 1 token ~= ¾ words
+    if not prompt:
+        return 0
+    est_tokens = 0
+    if isinstance(prompt, str):
+        est1 = len(prompt) / 4
+        est2 = len(_punc_regex.findall(prompt)) * 0.75
+        return round((1.5 * est1 + 0.5 * est2) / 2)
+    elif isinstance(prompt, list) and isinstance(prompt[0], int):
+        return len(prompt)
+    return est_tokens
