@@ -1,5 +1,5 @@
 """This Flask application is imported on tests.appsec.appsec_utils.gunicorn_server"""
-
+import ddtrace.auto  # noqa: F401  # isort: skip
 import copy
 import os
 import re
@@ -8,11 +8,12 @@ import subprocess  # nosec
 
 from flask import Flask
 from flask import Response
+from flask import jsonify
 from flask import request
+import urllib3
 from wrapt import FunctionWrapper
 
-
-import ddtrace.auto  # noqa: F401  # isort: skip
+import ddtrace
 from ddtrace import tracer
 from ddtrace.appsec._iast import ddtrace_iast_flask_patch
 from ddtrace.appsec._iast._taint_tracking._taint_objects_base import is_pyobject_tainted
@@ -22,6 +23,7 @@ from tests.appsec.iast_packages.packages.pkg_aiosignal import pkg_aiosignal
 from tests.appsec.iast_packages.packages.pkg_annotated_types import pkg_annotated_types
 from tests.appsec.iast_packages.packages.pkg_asn1crypto import pkg_asn1crypto
 from tests.appsec.iast_packages.packages.pkg_attrs import pkg_attrs
+from tests.appsec.iast_packages.packages.pkg_babel import pkg_babel
 from tests.appsec.iast_packages.packages.pkg_beautifulsoup4 import pkg_beautifulsoup4
 from tests.appsec.iast_packages.packages.pkg_cachetools import pkg_cachetools
 from tests.appsec.iast_packages.packages.pkg_certifi import pkg_certifi
@@ -95,12 +97,16 @@ from tests.appsec.iast_packages.packages.pkg_zipp import pkg_zipp
 import tests.appsec.integrations.flask_tests.module_with_import_errors as module_with_import_errors
 
 
+# Patch urllib3 since they are not patched automatically
+ddtrace.patch_all(urllib3=True)  # type: ignore
+
 app = Flask(__name__)
 app.register_blueprint(pkg_aiohttp)
 app.register_blueprint(pkg_aiosignal)
 app.register_blueprint(pkg_annotated_types)
 app.register_blueprint(pkg_asn1crypto)
 app.register_blueprint(pkg_attrs)
+app.register_blueprint(pkg_babel)
 app.register_blueprint(pkg_beautifulsoup4)
 app.register_blueprint(pkg_cachetools)
 app.register_blueprint(pkg_certifi)
@@ -173,6 +179,15 @@ app.register_blueprint(pkg_yarl)
 app.register_blueprint(pkg_zipp)
 
 
+def _weak_hash_vulnerability():
+    import _md5
+
+    m = _md5.md5()
+    m.update(b"Nobody inspects")
+    m.update(b" the spammish repetition")
+    m.digest()
+
+
 @app.route("/")
 def index():
     return "OK_index", 200
@@ -209,6 +224,14 @@ def view_cmdi_secure():
     return Response("OK")
 
 
+@app.route("/iast-unvalidated_redirect-header", methods=["GET"])
+def view_iast_unvalidated_redirect_insecure_header():
+    location = request.args.get("location")
+    response = Response("OK")
+    response.headers["Location"] = location
+    return response
+
+
 @app.route("/iast-header-injection-vulnerability", methods=["POST"])
 def iast_header_injection_vulnerability():
     header = request.form.get("header")
@@ -248,12 +271,7 @@ def iast_stacktrace_vulnerability():
 
 @app.route("/iast-weak-hash-vulnerability", methods=["GET"])
 def iast_weak_hash_vulnerability():
-    import _md5
-
-    m = _md5.md5()
-    m.update(b"Nobody inspects")
-    m.update(b" the spammish repetition")
-    m.digest()
+    _weak_hash_vulnerability()
     from ddtrace.internal import telemetry
 
     list_metrics_logs = list(telemetry.telemetry_writer._logs)
@@ -922,6 +940,25 @@ def test_flask_common_modules_patch_read():
     return Response(f"OK: {isinstance(copy_open, FunctionWrapper)}")
 
 
+@app.route("/returnheaders", methods=["GET"])
+def return_headers(*args, **kwargs):
+    headers = {}
+    for key, value in request.headers.items():
+        headers[key] = value
+    return jsonify(headers)
+
+
+@app.route("/vulnerablerequestdownstream", methods=["GET"])
+def vulnerable_request_downstream():
+    _weak_hash_vulnerability()
+    # Propagate the received headers to the downstream service
+    http_poolmanager = urllib3.PoolManager(num_pools=1)
+    # Sending a GET request and getting back response as HTTPResponse object.
+    response = http_poolmanager.request("GET", "http://localhost:8050/returnheaders")
+    http_poolmanager.clear()
+    return Response(response.data)
+
+
 if __name__ == "__main__":
     env_port = os.getenv("FLASK_RUN_PORT", 8000)
     debug = asbool(os.getenv("FLASK_DEBUG", "false"))
