@@ -318,7 +318,7 @@ def test_lcel_chain_batch(langchain_core, langchain_openai, openai_chat_completi
         "latest": LANGCHAIN_VERSION >= (0, 3),
     },
 )
-def test_lcel_chain_nested(langchain_core, langchain_openai, request_vcr):
+def test_lcel_chain_nested(langchain_core, langchain_openai, openai_url):
     """
     Test that invoking a nested chain will result in a 4-span trace with a root
     RunnableSequence span (complete_chain), then another RunnableSequence (chain1) +
@@ -329,15 +329,13 @@ def test_lcel_chain_nested(langchain_core, langchain_openai, request_vcr):
         "what country is the city {city} in? respond in {language}"
     )
 
-    model = langchain_openai.ChatOpenAI()
+    model = langchain_openai.ChatOpenAI(base_url=openai_url)
 
     chain1 = prompt1 | model | langchain_core.output_parsers.StrOutputParser()
     chain2 = prompt2 | model | langchain_core.output_parsers.StrOutputParser()
 
     complete_chain = {"city": chain1, "language": itemgetter("language")} | chain2
-
-    with request_vcr.use_cassette("lcel_openai_chain_nested.yaml"):
-        complete_chain.invoke({"person": "Spongebob Squarepants", "language": "Spanish"})
+    complete_chain.invoke({"person": "Spongebob Squarepants", "language": "Spanish"})
 
 
 @pytest.mark.asyncio
