@@ -1,7 +1,7 @@
 from unittest import TestCase
 
 from ddtrace import config
-from ddtrace.settings import IntegrationConfig
+from ddtrace.settings.integration import IntegrationConfig
 from ddtrace.trace import Pin
 
 
